from loguru import logger


def epub(src: str, dst: str) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == 'pdf':
        epub_to_pdf(src, dst)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def epub_to_pdf(src: str, dst: str) -> None:
    converter = EpubToPDF(
        src_path=src,
        dst_path=dst,
    )
    converter.convert()
    return


"""
Code for converting the EPUB to PDF.
"""

import os
import zipfile
import io
import img2pdf
from PIL import Image
from PyPDF2 import PdfWriter, PdfReader
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import letter
from lxml import etree


class EpubToPDF:
    def __init__(self, src_path, dst_path):
        self.src_path = src_path
        self.dst_path = dst_path

    def convert(self):
        with zipfile.ZipFile(self.src_path, 'r') as epub:
            # Extract the contents of the EPUB file
            epub.extractall(os.path.splitext(self.src_path)[0])

            # Find the OPF file
            opf_file = self.find_opf_file(epub)

            # Parse the OPF file
            opf_path = os.path.join(os.path.splitext(self.src_path)[0], opf_file)
            tree = etree.parse(opf_path)
            root = tree.getroot()

            # Namespace dictionary
            ns = {'opf': 'http://www.idpf.org/2007/opf', 'xhtml': 'http://www.w3.org/1999/xhtml'}

            # Find the manifest items
            manifest_items = root.findall('.//opf:manifest/opf:item', namespaces=ns)

            # Create a PDF writer
            pdf_writer = PdfWriter()

            # Iterate over the manifest items
            for item in manifest_items:
                # Check if the item is an XHTML file
                if item.attrib['media-type'] == 'application/xhtml+xml':
                    xhtml_path = os.path.join(os.path.dirname(opf_path), item.attrib['href'])
                    self.add_xhtml_to_pdf(xhtml_path, pdf_writer)
                # Check if the item is an image
                elif item.attrib['media-type'].startswith('image/'):
                    image_path = os.path.join(os.path.dirname(opf_path), item.attrib['href'])
                    self.add_image_to_pdf(image_path, pdf_writer)

            # Write the PDF to the destination path
            with open(self.dst_path, 'wb') as pdf_file:
                pdf_writer.write(pdf_file)

    def find_opf_file(self, epub):
        # Find the OPF file in the EPUB
        for file in epub.namelist():
            if file.endswith('.opf'):
                return file
        raise Exception('OPF file not found in the EPUB.')

    def add_xhtml_to_pdf(self, xhtml_path, pdf_writer):
        # Parse the XHTML file
        tree = etree.parse(xhtml_path)
        root = tree.getroot()

        # Create a BytesIO object to write the PDF content
        packet = io.BytesIO()

        # Create a canvas object
        c = canvas.Canvas(packet, pagesize=letter)

        # Render the XHTML content on the canvas
        for element in root.iter():
            if element.text:
                c.drawString(100, 700, element.text)

        # Save the canvas to the BytesIO object
        c.showPage()
        c.save()

        # Get the PDF content from the BytesIO object
        packet.seek(0)
        new_pdf = PdfReader(packet)

        # Add the new page to the PDF writer
        pdf_writer.add_page(new_pdf.pages[0])

    def add_image_to_pdf(self, image_path, pdf_writer):
        # Open the image file
        with open(image_path, 'rb') as image_file:
            # Create an image object
            image = Image.open(image_file)

            # Convert the image to bytes
            image_bytes = io.BytesIO()
            image.save(image_bytes, format='PNG')

            # Create a new PDF page with the image
            pdf_bytes = img2pdf.convert(image_bytes.getvalue())
            pdf_page = PdfReader(io.BytesIO(pdf_bytes)).pages[0]

            # Add the new page to the PDF writer
            pdf_writer.add_page(pdf_page)



from sys import stderr

from click import group
from click import version_option
from loguru import logger

from pd.config.config import config
from pd.conv.conv import conv
from pd.down.down import down
from pd.ec2.ec2 import ec2
from pd.edit.edit import edit
from pd.env.env import env
from pd.init.init import init
from pd.nginx.nginx import nginx
from pd.version import __version__


@group()
@version_option(__version__, '-v', '--version', help='Show the version and exit')
def cli():
    pass


cli.add_command(config)
cli.add_command(conv)
cli.add_command(down)
cli.add_command(ec2)
cli.add_command(edit)
cli.add_command(env)
cli.add_command(init)
cli.add_command(nginx)


# cli.add_command(sync)


def main():
    logger.remove(0)
    logger.add(stderr, level="DEBUG")
    cli()


if __name__ == "__main__":
    main()



from PIL import Image
from moviepy.editor import ImageClip

from loguru import logger


def to_png(src: str) -> str:
    ext = src.split(".")[-1]
    dst = src.replace(f".{ext}", ".png")

    with Image.open(src) as image:
        image.save(dst, "PNG")

    logger.debug(f"Converted {src} to {dst}")

    return dst


def png(src: str, dst: str, duration: int) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == 'mp4':
        png_to_mp4(src, dst, duration)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def png_to_mp4(src: str, dst: str, duration: int) -> None:
    logger.debug(f"Converting {src} to {dst} of {duration} seconds")

    clip = ImageClip(src, duration=duration)
    clip.write_videofile(dst, fps=24, codec='libx264')
    return



from os import getenv
from os import path as os_path
from typing import List

from boto3 import client as boto3_client
from click import group
from click import option
from loguru import logger

from ..config.config import add_instances
from ..config.config import load_pdconfig
from ..config.config import remove_instance
from ..config.config import update_instance
from ..model import Instance
from ..model import PD_CONFIG_FILE

AWS_ACCESS_KEY_ID = getenv('AWS_ACCESS_KEY_ID')
AWS_SECRET_ACCESS_KEY = getenv('AWS_SECRET_ACCESS_KEY')
AWS_DEFAULT_REGION = getenv('AWS_DEFAULT_REGION')


@group(name='ec2', help="Manage EC2 instances")
def ec2():
    logger.debug("ec2")
    if (
            AWS_ACCESS_KEY_ID is None or AWS_ACCESS_KEY_ID == '' or
            AWS_SECRET_ACCESS_KEY is None or AWS_SECRET_ACCESS_KEY == '' or
            AWS_DEFAULT_REGION is None or AWS_DEFAULT_REGION == ''
    ):
        print("Please set AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, and AWS_DEFAULT_REGION")
    return


@ec2.command(help="Launch an EC2 instance")
@option('-n', '--name', type=str, required=True, prompt=True,
        help="Name of the EC2 instance e.g. ec2-test")
@option('-c', '--count', type=int, default=1,
        help="Number of instances to launch")
@option('-p', '--project', type=str, default="",
        help="Project name e.g /path/to/project")
def launch(name, count, project):
    logger.debug("ec2 launch")

    if count > 1:
        logger.warning(f"{count} instances are being launched...")
        input("Press Enter to continue...")
    if project != "":
        if not os_path.isabs(project):
            logger.error(f"Project path {project} is not absolute")
            return
        if not os_path.exists(project):
            logger.error(f"Project {project} does not exist on disk")
            return

    # Load config.
    config = load_pdconfig()

    if 'ec2' not in config or len(config['ec2']) == 0:
        print(f"EC2 config not found, please update {PD_CONFIG_FILE}[ec2]")
        return

    ec2_config = config['ec2']
    if all(k in ec2_config for k in ('launch-template-id', 'key-pair-path')) is False:
        print(f"EC2 config is missing fields, please update {PD_CONFIG_FILE}[ec2]")
        return

    launch_template_id = config['ec2']['launch-template-id']
    key_pair_name = config['ec2']['key-pair-path'].split('/')[-1]
    if key_pair_name.endswith('.pem'):
        key_pair_name = key_pair_name[:-4]

    instances = launch_instances(
        launch_template_id=launch_template_id,
        key_pair_name=key_pair_name,
        instance_name=name,
        count=count
    )
    if len(instances) == 0:
        print("Failed to launch EC2 instances")
        return

    if count == 1:
        print(f"Launched 1 EC2 Instance: {instances[0]}")
    else:
        print(f"Launched {count} EC2 Instances: {instances}")

    add_instances(
        instances=instances,
        project=None if project == "" else project
    )


@ec2.command(help="Connect to an EC2 instance")
@option('-i', '--instance-id', type=str, required=True, prompt=True,
        help="ID of the EC2 instance to connect to e.g. i-1234567890abcdef")
def connect(instance_id):
    logger.debug("ec2 connect")


@ec2.command(help="Allocate an Elastic IP Address")
@option('-i', '--instance-id', type=str, required=True, prompt=True,
        help="ID of the EC2 instance to allocate an EIP for e.g. i-1234567890abcdef")
@option('-p', '--project', type=str, default="",
        help="Project name e.g /path/to/project")
def allocate(instance_id, project):
    logger.debug("ec2 allocate")

    public_ip = create_and_associate_eip(instance_id)
    if public_ip == "":
        print("Failed to allocate EIP")
        return

    print(f"Allocated EIP: {public_ip}")

    update_instance(
        instance=Instance(id=instance_id, ip=public_ip),
        project=None if project == "" else project
    )


@ec2.command(help="Terminate an EC2 instance")
@option('-i', '--instance-id', type=str, required=True, prompt=True,
        help="ID of the EC2 instance to terminate e.g. i-1234567890abcdef")
@option('-p', '--project', type=str, default="",
        help="Project name e.g /path/to/project")
def terminate(instance_id, project):
    logger.debug("ec2 terminate")

    if project != "":
        if not os_path.isabs(project):
            logger.error(f"Project path {project} is not absolute")
            return
        if not os_path.exists(project):
            logger.error(f"Project {project} does not exist on disk")
            return

    logger.info(f"Terminating EC2 Instance {instance_id}")
    input("Press Enter to continue...")

    ec2_client = boto3_client('ec2')

    try:
        response = ec2_client.terminate_instances(
            InstanceIds=[instance_id]
        )
        instances_data = response['TerminatingInstances']
        if len(instances_data) == 0:
            print("Failed to terminate EC2 instance")
            return

        instance_id = instances_data[0]['InstanceId']
        logger.debug(f"Terminated 1 EC2 instance with ID: {instance_id}")
        print(f"Terminated 1 EC2 instance with ID: {instance_id}")
    except Exception as e:
        msg = f"Failed to terminate EC2 instance due to {e}"
        logger.error(msg)
        print("Failed to launch EC2 instance")

    remove_instance(
        instance_id=instance_id,
        project=None if project == "" else project
    )


def launch_instances(
        launch_template_id: str,
        key_pair_name: str,
        instance_name: str,
        count: int,
) -> List[Instance]:
    if count < 1:
        logger.error("Count must be greater than 0")
        return []

    ec2_client = boto3_client('ec2')

    try:
        # Launch an EC2 instance using the specified launch template, security group, and key pair
        response = ec2_client.run_instances(
            LaunchTemplate={'LaunchTemplateId': launch_template_id},
            KeyName=key_pair_name,
            MinCount=count,
            MaxCount=count,
            TagSpecifications=[
                {
                    'ResourceType': 'instance',
                    'Tags': [
                        {
                            'Key': 'Name',
                            'Value': instance_name
                        }
                    ]
                }
            ]
        )

        # Extract the instance ID from the response
        instances_data = response['Instances']
        instance_ids = [instance['InstanceId'] for instance in instances_data]

        instances: List[Instance] = []
        for instance_id in instance_ids:
            instances.append(Instance(id=instance_id, ip=""))

        logger.debug(f"Launched {len(instances)} instance: {instances}")
        return instances
    except Exception as e:
        msg = f"Failed to launch EC2 Instances due to {e}"
        logger.error(msg)
        return []


def create_and_associate_eip(instance_id: str) -> str:
    if instance_id is None or instance_id == "":
        logger.error("Instance ID is required")
        return ""

    ec2_client = boto3_client('ec2')

    # Allocate a new Elastic IP Address
    allocation = ec2_client.allocate_address(Domain='vpc')
    eip_address = allocation['PublicIp']
    logger.debug(f"Allocated EIP: {eip_address}")

    # Associate the EIP with the specified EC2 instance
    allocation_id = allocation['AllocationId']
    ec2_client.associate_address(
        InstanceId=instance_id,
        AllocationId=allocation_id
    )
    logger.debug(f"Associated EIP {eip_address} with EC2 Instance {instance_id}")

    return eip_address



from importlib import resources
from json import dump as json_dump
from json import dumps as json_dumps
from json import load as json_load
from os import path as os_path
from typing import Dict
from typing import List

from click import group
from click import option
from jinja2 import BaseLoader
from jinja2 import Environment
from jinja2 import FileSystemLoader
from loguru import logger

from ..model import Instance
from ..model import PD_CONFIG_FILE

jinja2_env = Environment(loader=FileSystemLoader('./templates'))


def load_pdconfig() -> Dict:
    if not os_path.exists(PD_CONFIG_FILE):
        return {"projects": []}

    with open(PD_CONFIG_FILE, 'r') as file:
        data = json_load(file)
        file.close()
        return data


def save_pdconfig(data: Dict):
    with open(PD_CONFIG_FILE, 'w') as file:
        json_dump(data, file, indent=4)
        file.close()
    logger.debug(f"Saved config to {PD_CONFIG_FILE}")
    return


@group(name='config', help="Manage your config")
def config():
    if not os_path.exists(PD_CONFIG_FILE):
        with open(PD_CONFIG_FILE, 'w') as file:
            json_dump({"projects": [], "ec2": {}}, file, indent=4)
            file.close()
    return


@config.command(help="Show the config")
@option('-n', '--name', type=str, default="", help="Name of the item")
def show(name: str):
    logger.debug("config show")
    data = load_pdconfig()

    projects_data = data['projects'] if 'projects' in data else []
    all_ec2_instances = data['ec2']['instances'] if 'ec2' in data and 'instances' in data['ec2'] else []

    if name == "":
        data_string = json_dumps(data, indent=4)
        print(data_string)
    else:
        for project in projects_data:
            if name in project or name in project['path']:
                project_instances = project['instances'] if 'instances' in project else []

                instances_out = ""
                count = 1
                for ec2_instance in all_ec2_instances:
                    if 'instance-id' in ec2_instance and ec2_instance['instance-id'] in project_instances:
                        instance_id = ec2_instance['instance-id']
                        instance_ip = ec2_instance['instance-ip']

                        instances_out += f'\n\t{count}. {{"id":{instance_id},"ip":{instance_ip}}}'
                        count += 1

                print(f"Project: {project['name']}\nPath: {project['path']}\nInstances: {instances_out}")
                return
    return


@config.command(help="Create config for a project")
@option('-t', '--type', type=str, prompt=True,
        help="Config type e.g. zsh, ")
@option('-p', '--project', type=str, prompt=True,
        help="Project path e.g. /path/to/project")
@option('-d', '--domain', type=str, default="",
        help="Domain name e.g. example.com")
def create(type: str, project: str, domain: str):
    logger.debug("config create")

    if type == "zsh":
        config_name = ".zshrc"
        config_path = os_path.expanduser(f"~/{config_name}")
        if not os_path.exists(config_path):
            logger.error(f"Config file {config_path} not found")
            return

        project_config: List[str] = []
        lines: List[str] = open(config_path, 'r').readlines()
        for line in lines:
            if line.startswith(f"export {project.upper()}"):
                logger.debug(f"Matched {line}")
                project_config.append(line.strip())

        # Load template
        template_str = resources.read_text('pd.config.templates', 'zshrc.j2')
        j2_env = Environment(loader=BaseLoader())
        template = j2_env.from_string(template_str)
        args = {
            'config': "\n".join(project_config),
        }
        if domain != "":
            args['domain_export'] = f'export SITE_NAME="{domain}"'
        output = template.render(**args)
        print("~/.zshrc")
        print(output)
    return


@config.command(help="Manage your projects")
@option('-a', '--add', type=str, default="",
        help="Add a project e.g. react,/path/to/project")
@option('-d', '--delete', type=str, default="",
        help="Delete a project e.g. /path/to/project")
def projects(add: str, delete: str):
    manage_projects(add, delete)
    return


def manage_projects(
        add: str = "",
        delete: str = "",
):
    data = load_pdconfig()

    if add is not None and add != "":
        logger.debug("projects add")

        try:
            ptype, path, name, full_path = validate_project(add)
        except Exception as e:
            logger.error(e)
            return

        for project in data['projects']:
            if project['path'] == full_path:
                logger.debug(f"Project {project['path']} already added")
                return

        data['projects'].append(
            {
                "type": ptype,
                "name": name,
                "path": full_path,
            }
        )

    if delete is not None and delete != "":
        logger.debug("projects delete")

        for project in data['projects']:
            if project['path'] == delete:
                logger.debug(f"Deleting {project['path']}")
                data['projects'].remove(project)
                return

        logger.error(f"Project {delete} not found")

    save_pdconfig(data)

    return


def validate_project(project_str: str):
    if ',' not in project_str:
        msg = f"Invalid format {project_str}, use <type>,<path>"
        raise Exception(msg)

    ptype, ppath = project_str.split(',', 1)

    valid_ptypes = ["react", "fastapi", "electron", "nextjs"]

    # Validate project type
    if ptype not in valid_ptypes:
        msg = f"Invalid project type {ptype}, use one of {valid_ptypes}"
        raise Exception(msg)

    # Validate project path
    path, name = (ppath.rsplit('/', 1) if '/' in ppath else (".", ppath))
    if path == ".":  # Expand path
        path = os_path.abspath(path)
    full_path = os_path.join(path, name)

    if not os_path.exists(full_path):
        msg = f"Path {full_path} does not exist, please create it first"
        raise Exception(msg)

    logger.debug(f"ptype: {ptype}")
    logger.debug(f"path: {path}")
    logger.debug(f"name: {name}")
    logger.debug(f"full_path: {full_path}")

    return ptype, path, name, full_path


def add_instances(
        instances: List[Instance],
        project: str = None
) -> None:
    if len(instances) == 0:
        logger.error("No Instances provided")
        return

    # Load config
    data = load_pdconfig()
    if 'ec2' not in data:
        data['ec2'] = {}
    if 'instances' not in data['ec2']:
        data['ec2']['instances'] = []

    # Add Instances to EC2
    for instance in instances:
        exists = False
        for pd_ec2_instance in data['ec2']['instances']:
            if 'instance-id' in pd_ec2_instance and pd_ec2_instance['instance-id'] == instance.id:
                exists = True
                break

        if not exists:
            logger.debug(f"Adding {instance.id} to {PD_CONFIG_FILE}[ec2][instances]")
            data['ec2']['instances'].append(
                {
                    "instance-id": instance.id,
                    "instance-ip": instance.ip,
                }
            )

    # Add Instances to Project
    if project is not None and project != "":
        for p in data['projects']:
            if p['path'] == project:
                if 'instances' not in p:
                    pd_project_instances: List[Dict[str, str]] = []
                    for instance in instances:
                        pd_project_instances.append(
                            {
                                "instance-id": instance.id,
                                "instance-ip": instance.ip,
                            }
                        )
                    p['instances'] = pd_project_instances
                    return

                for instance in instances:
                    exists = False
                    for pd_project_instance in p['instances']:
                        if 'instance-id' in pd_project_instance and pd_project_instance['instance-id'] == instance.id:
                            exists = True
                            break

                    if not exists:
                        logger.debug(f"Adding {instance.id} to {PD_CONFIG_FILE}[projects][{project}][instances]")
                        p['instances'].append(
                            {
                                "instance-id": instance.id,
                                "instance-ip": instance.ip,
                            }
                        )

    save_pdconfig(data)
    return


def update_instance(
        instance: Instance,
        project: str = None
) -> None:
    # Load config
    data = load_pdconfig()
    if 'ec2' not in data or 'instances' not in data['ec2']:
        logger.error(f"No instances found in {PD_CONFIG_FILE}[ec2][instances]")
        return

    # Add Instances to EC2
    for pd_ec2_instance in data['ec2']['instances']:
        if 'instance-id' in pd_ec2_instance and pd_ec2_instance['instance-id'] == instance.id:
            logger.debug(f"Updating {instance.id} in {PD_CONFIG_FILE}[ec2][instances]")

            data['ec2']['instances'].remove(pd_ec2_instance)
            data['ec2']['instances'].append(
                {
                    "instance-id": instance.id,
                    "instance-ip": instance.ip,
                }
            )

    # Add Instances to Project
    if project is not None and project != "":
        for p in data['projects']:
            if p['path'] == project:
                if 'instances' not in p:
                    logger.error(f"No instances found in {PD_CONFIG_FILE}[projects][{project}][instances]")
                    return

                for pd_project_instance in p['instances']:
                    if 'instance-id' in pd_project_instance and pd_project_instance['instance-id'] == instance.id:
                        logger.debug(f"Updating {instance.id} in {PD_CONFIG_FILE}[projects][{project}][instances]")

                        p['instances'].remove(pd_project_instance)
                        p['instances'].append(
                            {
                                "instance-id": instance.id,
                                "instance-ip": instance.ip,
                            }
                        )

    save_pdconfig(data)
    return


def remove_instance(
        instance_id: str,
        project: str = None
) -> None:
    if instance_id == "":
        logger.error("No instance ID provided")
        return

    # Load config
    data = load_pdconfig()

    # Remove Instance from EC2
    if 'ec2' in data and 'instances' in data['ec2']:
        for pd_ec2_instance in data['ec2']['instances']:
            if 'instance-id' in pd_ec2_instance and pd_ec2_instance['instance-id'] == instance_id:
                logger.debug(f"Removing {instance_id} from {PD_CONFIG_FILE}[ec2][instances]")

                data['ec2']['instances'].remove(pd_ec2_instance)

    # Remove Instance from Project
    if project is not None and project != "":
        for p in data['projects']:
            if 'path' in p and 'instances' in p and p['path'] == project:
                for pd_project_instance in p['instances']:
                    if 'instance-id' in pd_project_instance and pd_project_instance['instance-id'] == instance_id:
                        logger.debug(f"Removing {instance_id} from {PD_CONFIG_FILE}[projects][{project}][instances]")

                        p['instances'].remove(pd_project_instance)

    save_pdconfig(data)
    return



from os import path as os_path
from subprocess import run as os_run

from click import argument
from click import group
from click import option
from loguru import logger


@group(name='env', help="Manage environments")
def env():
    pass


@env.command(help="Show an environment")
@option('-n', '--name', type=str, required=True, prompt=True,
        help='Name of the environment e.g. zsh, vim, git')
def show(name):
    logger.debug("env view")

    env_name = {
        'zsh': 'zshrc',
        'vim': 'vimrc',
        'git': 'gitconfig',
        'tmux': 'tmux.conf',
    }
    if name not in env_name:
        logger.error(f"Unknown environment {name}")
        return

    env_path = os_path.expanduser(f"~/.{env_name[name]}")
    logger.debug(f"env_path: {env_path}")

    if not os_path.exists(env_path):
        logger.error(f"Environment {name} not found")
        return

    with open(env_path, 'r') as f:
        print(f.read())
    return


@env.command(help="Activate an environment")
@argument('lang', type=str, required=True)
@argument('tool', type=str, required=True)
def up(lang: str, tool: str):
    logger.debug("env up")

    command = ""
    match lang:
        case 'py':
            match tool:
                case 'venv':
                    command = "source venv/bin/activate"
                case 'conda':
                    command = "conda activate"
                case _:
                    logger.error(f"Unknown tool {tool}")
        case _:
            logger.error(f"Unknown language {lang}")

    logger.debug(f"Run: {command}")
    os_run(command, shell=True)
    return


@env.command(help="Deactivate an environment")
@argument('lang', type=str, required=True)
@argument('tool', type=str, required=True)
def down(lang: str, tool: str):
    logger.debug("env down")

    command = ""
    match lang:
        case 'py':
            match tool:
                case 'venv':
                    command = "deactivate"
                case 'conda':
                    command = "conda deactivate"
                case _:
                    logger.error(f"Unknown tool {tool}")
        case _:
            logger.error(f"Unknown language {lang}")

    logger.debug(f"Run: {command}")
    os_run(command, shell=True)
    return


@env.command(help="Sync an environment")
@option('-n', '--name', type=str, required=True, prompt=True,
        help='Name of the environment e.g. zsh, vim, git')
def sync(name):
    logger.debug("env sync")
    return



from importlib import resources
from re import compile as re_compile

from click import group
from click import option
from jinja2 import BaseLoader
from jinja2 import Environment
from loguru import logger


@group(name='nginx', help="Manage nginx files")
def nginx():
    pass


@nginx.command(help="Generate nginx config")
@option('-h', '--host', type=str, required=True, default='localhost',
        help="Host to proxy to e.g. localhost")
@option('-p', '--port', type=int, required=True, default=80,
        help="Port to proxy to e.g. 80")
@option('-d', '--domain', type=str, required=True, default='example.com',
        help="Domain to use for nginx config e.g. example.com")
@option('-s', '--static', type=str, default='',
        help="Static resources path e.g. /path/to/static")
def generate(host: str, port: int, domain: str, static: str):
    logger.debug("nginx generate")

    re_scheme = re_compile(r'^https?://')

    scheme = 'http'
    if host.startswith('https://'):
        scheme = 'https'
    host = re_scheme.sub('', host)
    if port < 0 or port > 65535:
        logger.error("invalid port")
        return
    domain = re_scheme.sub('', domain)

    template_name = 'simple.com' if static == '' else 'static.com'
    template_str = resources.read_text('pd.nginx.templates', template_name)

    logger.debug(f"scheme: {scheme}")
    logger.debug(f"host: {host}")
    logger.debug(f"port: {port}")
    logger.debug(f"domain: {domain}")
    logger.debug(f"template_str: {template_str}")

    j2_env = Environment(loader=BaseLoader())
    template = j2_env.from_string(template_str)
    args = {
        'scheme': scheme,
        'host': host,
        'port': port,
        'domain': domain,
    }
    if static != '':
        args['static'] = static
    output = template.render(**args)
    print(f"/etc/nginx/sites-available/{domain}")
    print(output)
    return



from click import group
from click import option
from loguru import logger
from pytube import YouTube
from youtube_transcript_api import YouTubeTranscriptApi
from re import sub as re_sub


@group(name='down', help="Download from the internet")
def down():
    pass


@down.command(help="Download a YouTube video")
@option('-l', '--link', type=str, required=True, prompt=True,
        help="Link to the YouTube video (e.g. https://www.youtube.com/watch?v=...)")
@option('-f', '--format', type=str, required=True, prompt=True,
        help="Format to download as (e.g. mp4, mp3, txt)")
def youtube(link: str, format: str):
    logger.debug("down video")

    if format == 'mp4':
        yt = YouTube(link)
        stream = yt.streams.filter(progressive=True, file_extension='mp4').order_by('resolution').desc().first()
        stream.download()
    elif format == 'webm':
        yt = YouTube(link)
        stream = yt.streams.filter(only_audio=True).order_by('abr').desc().first()
        stream.download()
    elif format == 'txt':
        title, script = get_script(link)
        with open(f"{title}.txt", 'w') as f:
            f.write(script)
    else:
        logger.error(f"Unsupported format {format}")


def get_script(url: str) -> (str, str):
    yt = YouTube(url)
    title = re_sub(r'[\/:*?"<>|]', '', yt.title)

    if 'youtube.com' in url:
        url = url.split('=')[-1]

    subtitles = YouTubeTranscriptApi.get_transcript(url)

    script = " ".join([f"{sub['text']}" for sub in subtitles])
    return title, script


if __name__ == '__main__':
    link = f"https://www.youtube.com/watch?v=DxREm3s1scA"
    t, s = get_script(link)
    logger.info(f"{t}: {s}")



from loguru import logger
from os import getenv
from openai import OpenAI
from typing import Dict, Literal


def txt(src: str, dst: str, opts: Dict[str, str]) -> None:
    dst_format = dst.split(".")[-1]

    if dst_format == 'mp3':
        txt_to_mp3(src, dst, opts)
    else:
        logger.error(f"Unsupported format {dst_format}")
    return


def txt_to_mp3(src: str, dst: str, opts: Dict[str, str]) -> None:
    OPENAI_API_KEY = getenv("OPENAI_API_KEY")
    if OPENAI_API_KEY is None:
        logger.error("OPENAI_API_KEY is not set")
        return

    client = OpenAI()

    args = {"model": "tts-1", "voice": opts.get("voice", "echo"), "input": src}
    response = client.audio.speech.create(**args)

    # Save to disk.
    with open(dst, "wb") as file:
        file.write(response.read())

    return



from os import path as os_path
from typing import List
from typing import Tuple

from click import group
from click import option
from loguru import logger
from PIL import Image
from PIL import ImageDraw
from PIL import ImageFilter


@group(name='edit', help="Edit a file")
def edit():
    pass


@edit.command(help="Round a png image")
@option('-p', '--path', type=str, required=True, prompt=True,
        help="Input image file path")
@option('-r', '--radius', type=str, default="20%", show_default=True,
        help="Radius (pixels or percentage)")
def round(path: str, radius: str):
    logger.debug("round")

    img = Image.open(path).convert("RGBA")
    width, height = img.size

    radius_px = radius_to_pixels(radius, width, height)
    rounded_img = round_corners(img, radius_px)

    # Generate new filename
    file_name, file_ext = os_path.splitext(path)
    if width == height:
        new_file_name = f"{file_name}{width}{file_ext}"
    else:
        new_file_name = f"{file_name}{width}x{height}{file_ext}"

    # Save the rounded image
    rounded_img.save(new_file_name, 'PNG')
    logger.info(f"Rounded image saved as {new_file_name}")


@edit.command(help="Generate logos from a png image")
@option('-p', '--path', type=str, required=True, prompt=True,
        help="Input image file path")
@option('-t', '--template', type=str, default="Generic=(16, 32, 48, 128, 256, 512, 1024)", show_default=True,
        help="Template for logo sizes")
@option('-r', '--radius', type=str, default="", show_default=True,
        help="Radius (pixels or percentage)")
def logos(path: str, template: str, radius: str):
    logger.debug("logo")

    # Open the original image
    original_image = Image.open(path).convert("RGBA")
    width, height = original_image.size

    # Ensure the image is square
    if width != height:
        logger.warning("Image is not square. Cropping to make it square.")
        size = min(width, height)
        left = (width - size) // 2
        top = (height - size) // 2
        right = left + size
        bottom = top + size
        original_image = original_image.crop((left, top, right, bottom))

    # Get sizes from template
    template = "Generic" if "Generic" in template else template
    sizes = get_sizes_from_template(template)

    # Check if upscaling is needed
    max_size = max(sizes)
    if original_image.size[0] < max_size:
        logger.info(f"Upscaling image to {max_size}x{max_size}")
        original_image = high_quality_resize(original_image, (max_size, max_size))

    if radius:
        radius_px = radius_to_pixels(radius, original_image.size[0], original_image.size[1])
        logger.info(f"Rounding corners with radius {radius_px}")
        original_image = round_corners(original_image, radius_px)

    # Generate all sizes
    for size in sizes:
        resized_image = high_quality_resize(original_image, (size, size))

        # Generate output filename
        file_name, file_ext = os_path.splitext(path)
        output_path = f"{file_name}{size}{file_ext}"

        # Save the resized image
        resized_image.save(output_path)
        logger.debug(f"Generated logo: {output_path}")

    sizes = [f"{size}x{size}" for size in sizes]
    logger.info(f"{len(sizes)} logos ({', '.join(sizes)}) and saved in the same directory")


@edit.command(help="Iconify a png image")
@option('-p', '--path', type=str, required=True, prompt=True,
        help="Input image file path")
@option('-s', '--sizes', type=list, default=[16, 32, 48], show_default=True,
        help="Sizes for icon")
def favicon(path: str, sizes: List[int]):
    logger.debug("iconify")

    original_image = Image.open(path).convert("RGBA")

    icon_sizes = []
    for size in sizes:
        resized_image = high_quality_resize(original_image, (size, size))
        icon_sizes.append(resized_image)

    output_path = f"{os_path.splitext(path)[0]}.ico"
    icon_sizes[0].save(
        output_path,
        format='ICO',
        sizes=[(size, size) for size in sizes],
        append_images=icon_sizes[1:]
    )

    logger.info(f"Icon created successfully: {output_path}")
    logger.info(f"Icon sizes: {sizes}")


def radius_to_pixels(radius: str, width: int, height: int) -> int:
    if not radius:
        return 0
    if radius.endswith('%'):
        percentage = float(radius[:-1]) / 100
        return int(min(width, height) * percentage / 2)
    else:
        return int(radius)


def round_corners(img: Image, radius: int) -> Image:
    circle = Image.new('L', (radius * 2, radius * 2), 0)
    draw = ImageDraw.Draw(circle)
    draw.ellipse((0, 0, radius * 2, radius * 2), fill=255)
    alpha = Image.new('L', img.size, 255)
    w, h = img.size
    alpha.paste(circle.crop((0, 0, radius, radius)), (0, 0))
    alpha.paste(circle.crop((0, radius, radius, radius * 2)), (0, h - radius))
    alpha.paste(circle.crop((radius, 0, radius * 2, radius)), (w - radius, 0))
    alpha.paste(circle.crop((radius, radius, radius * 2, radius * 2)), (w - radius, h - radius))
    img.putalpha(alpha)
    return img


def high_quality_resize(image: Image, size: Tuple[int, int]) -> Image:
    factor = 8
    img = image.copy()
    img.thumbnail((size[0] * factor, size[1] * factor), Image.LANCZOS)
    img = img.filter(ImageFilter.UnsharpMask(radius=2, percent=150, threshold=3))
    img = img.resize(size, Image.LANCZOS)
    return img


def get_sizes_from_template(template: str) -> List[int]:
    if template == "iOS":
        return [40, 60, 80, 120, 180]
    elif template == "Android":
        return [36, 48, 64, 72, 96, 144, 192]
    elif template == "macOS":
        return [16, 32, 128, 256, 512]
    elif template == "Chrome":
        return [16, 19, 32, 38, 48, 128]
    elif template == "Generic":
        return [16, 32, 48, 128, 256, 512, 1024]
    else:
        raise ValueError(f"Unsupported template: {template}")



from os import path as os_path

from loguru import logger
from setuptools import find_packages
from setuptools import setup


def read_long_description():
    with open('README.md', 'r') as f:
        long_description = f.read()
    return long_description


def read_version():
    version_file = os_path.join(os_path.dirname(__file__), 'pd', 'version.py')
    with open(version_file) as file:
        exec(file.read())
    version = locals()['__version__']
    logger.debug(f"Building {PACKAGE_NAME} v{version}")
    return version

def get_requirements():
    with open('requirements.txt') as f:
        return f.read().splitlines()


PACKAGE_NAME = 'zf-pd'
PACKAGE_VERSION = read_version()
AUTHOR_NAME = 'zeffmuks'
AUTHOR_EMAIL = 'zeffmuks@gmail.com'

setup(
    name=PACKAGE_NAME,
    version=PACKAGE_VERSION,
    author=AUTHOR_NAME,
    author_email=AUTHOR_EMAIL,
    description='pd supercharges your development workflows',
    long_description=read_long_description(),  # This is the important part
    long_description_content_type='text/markdown',  # This tells PyPI the content is in Markdown
    include_package_data=True,
    package_data={
        'pd.config': ['templates/*'],
        'pd.init': ['templates/pd-fastapi/*', 'templates/pd-react/*', 'templates/pd-nextjs/*',
                    'templates/pd-electron/*'],
        'pd.nginx': ['templates/*'],
    },
    install_requires=get_requirements(),
    packages=find_packages(
        include=['pd', 'pd.*'],
        exclude=['venv', 'venv.*']
    ),
    entry_points={
        'console_scripts': [
            'pd=pd.__main__:main'  # Invokes pd/__main__.py::main()
        ]
    },
)
